﻿namespace Interval.CZ.Web.UI.WebControls
{
  using System;
  using System.Web;
  using System.Web.UI;
  using System.Web.UI.WebControls;


  /// <summary>
  /// Enum of pagination modes
  /// </summary>
  public enum PaginationMode
  {
    /// <summary>
    ///
    /// </summary>
    AllControls,
    /// <summary>
    ///
    /// </summary>
    NextPrev,
    /// <summary>
    ///
    /// </summary>
    Numeric
  }

  /// <summary>
  /// Enum of pagination modes
  /// </summary>
  public enum PaginationNumericRepeatDirection
  {
    /// <summary>
    ///
    /// </summary>
    Horizontal,
    /// <summary>
    ///
    /// </summary>
    Vertical
  }


  /// <summary>
  /// Pagination control
  /// </summary>
  public class Pagination : System.Web.UI.Control, INamingContainer
  {
    protected System.Web.UI.WebControls.Button btnGo = new System.Web.UI.WebControls.Button();
    protected System.Web.UI.WebControls.Button btnPrev = new System.Web.UI.WebControls.Button();
    protected System.Web.UI.WebControls.Button btnNext = new System.Web.UI.WebControls.Button();
    protected System.Web.UI.WebControls.RadioButtonList rblNumeric = new System.Web.UI.WebControls.RadioButtonList();
    protected System.Web.UI.WebControls.Label lblNextPrev = new System.Web.UI.WebControls.Label();
    protected System.Web.UI.WebControls.Label lblGo = new System.Web.UI.WebControls.Label();

    private Int32 _virtualItemCount = 0, _pageCount = 0, _selectedPageIndex = 0;
    private Int32 _pageSize = 10, _pageButtonCount = 10;
    private PaginationNumericRepeatDirection _numericRepeatDirection = PaginationNumericRepeatDirection.Horizontal;
    private PaginationMode _mode = PaginationMode.AllControls;

    private static object EventChangeKey = new object();


    public Pagination(): base()
    {
      lblNextPrev.Visible= false;
      btnPrev.Text = "Prev";
      btnPrev.CausesValidation = true;
      btnPrev.Enabled = false;
      btnNext.Text = "Next";
      btnNext.CausesValidation = true;
      btnNext.Enabled = false;
      rblNumeric.Visible = false;
      rblNumeric.AutoPostBack = true;
      rblNumeric.RepeatDirection = RepeatDirection.Horizontal;
      rblNumeric.RepeatLayout = RepeatLayout.Flow;
      rblNumeric.CausesValidation = true;
      lblGo.Visible = false;
      btnGo.Text = "Go";
      btnGo.CausesValidation = true;
    }

    public event EventHandler SelectedPageIndexChanged
    {
      add
      {
        Events.AddHandler(EventChangeKey, value);
      }

      remove
      {
        Events.RemoveHandler(EventChangeKey, value);
      }
    }

    protected virtual void OnSelectedPageIndexChange(EventArgs e)
    {
      _calculatePageCount();
      EventHandler eh = (EventHandler) Events[EventChangeKey];
      if (eh != null)
        eh(this, e);
    }

    /// <summary>
    /// Go button text
    /// </summary>
    public String GoToPageText
    {
      get
      {
        return btnGo.Text;
      }
      set
      {
        btnGo.Text = value;
      }
    }

    /// <summary>
    /// Go button tooltip
    /// </summary>
    public String GoToPageToolTip
    {
      get
      {
        return btnGo.ToolTip;
      }
      set
      {
        btnGo.ToolTip = value;
      }
    }

    /// <summary>
    /// Numeric pages tooltip
    /// </summary>
    public String NumericPagesToolTip
    {
      get
      {
        return rblNumeric.ToolTip;
      }
      set
      {
        rblNumeric.ToolTip = value;
      }
    }

    /// <summary>
    /// Next page button text
    /// </summary>
    public String NextPageText
    {
      get
      {
        return btnNext.Text;
      }
      set
      {
        btnNext.Text = value;
      }
    }

    /// <summary>
    /// Next page button tooltip
    /// </summary>
    public String NextPageToolTip
    {
      get
      {
        return btnNext.ToolTip;
      }
      set
      {
        btnNext.ToolTip = value;
      }
    }

    /// <summary>
    /// Previous page button text
    /// </summary>
    public String PrevPageText
    {
      get
      {
        return btnPrev.Text;
      }
      set
      {
        btnPrev.Text = value;
      }
    }

    /// <summary>
    /// Previous page button tooltip
    /// </summary>
    public String PrevPageToolTip
    {
      get
      {
        return btnPrev.ToolTip;
      }
      set
      {
        btnPrev.ToolTip = value;
      }
    }


    /// <summary>
    /// Button CSS class
    /// </summary>
    public String ButtonCssClass
    {
      get
      {
        return lblNextPrev.CssClass;
      }
      set
      {
        lblNextPrev.CssClass = lblGo.CssClass = value;
      }
    }

    /// <summary>
    /// RadioButton CSS class
    /// </summary>
    public String RadioButtonCssClass
    {
      get
      {
        return rblNumeric.CssClass;
      }
      set
      {
        rblNumeric.CssClass = value;
      }
    }

    /// <summary>
    /// Gets or sets the virtual number of items in the Pagination control.
    /// </summary>
    public Int32 VirtualItemCount
    {
      get
      {
        object o = this.ViewState["VirtualItemCount"];
        if (o != null)
         return (Int32) o;
        else
         return _virtualItemCount;
      }
      set
      {
        this.ViewState["VirtualItemCount"] = value;
        _virtualItemCount = value;
      }

    }

    /// <summary>
    /// Gets or sets the index of the currently displayed page.
    /// </summary>
    public Int32 SelectedPageIndex
    {
      get
      {
        object o = this.ViewState["SelectedPageIndex"];
        if (o != null)
         return (Int32) o;
        else
         return _selectedPageIndex;
      }
      set
      {
        this.ViewState["SelectedPageIndex"] = value;
        _selectedPageIndex = value;
      }
    }

    /// <summary>
    /// Gets total number of pages to pagination.
    /// </summary>
    public Int32 PageCount
    {
      get
      {
         _calculatePageCount();
         return _pageCount;
      }
    }

    /// <summary>
    /// Gets or sets the number of items to display on a single page of the paginated control.
    /// </summary>
    public Int32 PageSize
    {
      get
      {
        object o = this.ViewState["PageSize"];
        if (o != null)
         return (Int32) o;
        else
         return _pageSize;
      }
      set
      {
        this.ViewState["PageSize"] = value;
        _pageSize = value;
      }
    }

    /// <summary>
    /// Gets or sets the number of numeric buttons to display concurrently
    /// </summary>
    public Int32 PageButtonCount
    {
      get
      {
        object o = this.ViewState["PageButtonCount"];
        if (o != null)
         return (Int32) o;
        else
         return _pageButtonCount;
      }
      set
      {
        this.ViewState["PageButtonCount"] = value;
        _pageButtonCount = value;
      }
    }

    /// <summary>
    /// Gets or sets the direction that the radio buttons within the group are displayed.
    /// </summary>
    public PaginationNumericRepeatDirection NumericRepeatDirection
    {
      get { return _numericRepeatDirection; }
      set { _numericRepeatDirection = value; }
    }

    /// <summary>
    /// Gets or sets a value that specifies whether the pager element displays buttons that link to the next and previous page, or numeric radio buttons.
    /// </summary>
    public PaginationMode Mode
    {
      get { return _mode; }
      set { _mode = value; }
    }

    /// <summary>
    /// Gets or sets a value indicating whether validation is performed when the control is clicked.
    /// </summary>
    public Boolean CausesValidation
    {
      get
      {
        return btnPrev.CausesValidation;
      }
      set
      {
        btnPrev.CausesValidation = btnNext.CausesValidation = btnGo.CausesValidation = rblNumeric.CausesValidation = value;
      }
    }

    private void _calculatePageCount()
    {
      if (_virtualItemCount > 0 && _pageSize > 0)
      {
        _pageCount = _virtualItemCount/_pageSize;
        if (_virtualItemCount %_pageSize > 0)
          _pageCount++;
      }   // Math.Ceiling(_virtualItemCount/_pageSize);
      else
        _pageCount = 0;
    }

    private void Page_Load(object sender, System.EventArgs e)
    {
      if (this.Page.IsPostBack)
      {
        if (this.ViewState["SelectedPageIndex"] != null)
          _selectedPageIndex = (Int32) this.ViewState["SelectedPageIndex"];
        if (this.ViewState["VirtualItemCount"] != null)
          _virtualItemCount =  (Int32) this.ViewState["VirtualItemCount"];
      }
    }

    private void btnPrev_Click(object sender, System.EventArgs e)
    {
      if (_selectedPageIndex > 0)
      {
        _selectedPageIndex--;
        ViewState["SelectedPageIndex"] = _selectedPageIndex;
        OnSelectedPageIndexChange(new EventArgs());
      }
    }

    private void btnNext_Click(object sender, System.EventArgs e)
    {
      _calculatePageCount();
      if (_selectedPageIndex+1 < _pageCount)
      {
        _selectedPageIndex++;
        ViewState["SelectedPageIndex"] = _selectedPageIndex;
        OnSelectedPageIndexChange(new EventArgs());
      }
    }

    private void rblNumeric_SelectedIndexChanged(object sender, System.EventArgs e)
    {
      // set pageindex
      Page.Validate();
      _selectedPageIndex = Int32.Parse(rblNumeric.SelectedValue);
      ViewState["SelectedPageIndex"] = _selectedPageIndex;
      OnSelectedPageIndexChange(new EventArgs());
    }



    override protected void OnPreRender(EventArgs e)
    {
      if (_virtualItemCount > 0)
      {
        _calculatePageCount();

        if (_selectedPageIndex < 1)
          btnPrev.Enabled = false;
        else
          btnPrev.Enabled = true;

        if (_selectedPageIndex+1 < _pageCount)
          btnNext.Enabled = true;
        else
        {
          _selectedPageIndex = _pageCount-1;
          btnNext.Enabled = false;
        }
      }
      else
      {
        btnPrev.Enabled = false;
        btnNext.Enabled = false;
      }

      lblNextPrev.Visible = true;

      if (_mode == PaginationMode.Numeric)
      {
        lblNextPrev.Visible = false;
      }



      rblNumeric.Visible = false;
      rblNumeric.Enabled = false;
      btnGo.Enabled = false;
      lblGo.Visible = false;
      if (_mode != PaginationMode.NextPrev)
      {
//        _calculatePageCount();

        rblNumeric.Visible = true;
        rblNumeric.Items.Clear();

        Int32 currentSetFirstItemValue = ((_selectedPageIndex/_pageButtonCount) * _pageButtonCount);
        Int32 actualItemValue = currentSetFirstItemValue;


        if (actualItemValue >= _pageButtonCount)
          rblNumeric.Items.Add(new ListItem("…",(actualItemValue-1).ToString()));

        while (actualItemValue < (currentSetFirstItemValue + _pageButtonCount) && actualItemValue < _pageCount)
        {
          rblNumeric.Items.Add(new ListItem((actualItemValue+1).ToString(),actualItemValue.ToString()));
          actualItemValue++;
        }
        if (actualItemValue < _pageCount)
          rblNumeric.Items.Add(new ListItem("…",actualItemValue.ToString()));

        if (rblNumeric.Items.Count > 0)
        {
          if (_selectedPageIndex > 0)
            rblNumeric.SelectedValue = _selectedPageIndex.ToString();
          else
            rblNumeric.SelectedIndex = 0;
          if (rblNumeric.Items.Count > 1)
          {
            btnGo.Enabled = true;
            rblNumeric.Enabled = true;
          }
        }
        else
        {
          rblNumeric.Items.Add(new ListItem("1","0"));
          rblNumeric.SelectedIndex = 0;
        }
        lblGo.Visible = true;
      }
      base.OnPreRender(e);
    }

    protected override void CreateChildControls()
    {
      this.Controls.Add(lblNextPrev);
      lblNextPrev.Controls.Add(btnPrev);
      lblNextPrev.Controls.Add(new LiteralControl("&nbsp;"));
      lblNextPrev.Controls.Add(btnNext);
      this.Controls.Add(rblNumeric);
      this.Controls.Add(new LiteralControl("\n<noscript>"));
      this.Controls.Add(lblGo);
      lblGo.Controls.Add(btnGo);
      this.Controls.Add(new LiteralControl("</noscript>"));
      this.rblNumeric.SelectedIndexChanged += new System.EventHandler(this.rblNumeric_SelectedIndexChanged);
      this.btnPrev.Click += new System.EventHandler(this.btnPrev_Click);
      this.btnNext.Click += new System.EventHandler(this.btnNext_Click);
    }

    override protected void OnInit(EventArgs e)
    {
      this.Load += new System.EventHandler(this.Page_Load);
    }
  }
}